package apis

import (
	v1 "opendev.org/x/networking-spp/operator/pkg/apis/spp/v1"
)

func init() {
	// Register the types with the Scheme so the components can map objects to GroupVersionKinds and back
	AddToSchemes = append(AddToSchemes, v1.SchemeBuilder.AddToScheme)
}
