# Copyright (c) 2017 NTT
# All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

from tempest.test_discover import plugins


class NetworkingSppTempestPlugin(plugins.TempestPlugin):
    def get_opt_lists(self):
        return []

    def load_tests(self):
        # .../networking-spp/networking_spp/tests/tempest/plugin.py
        #     top_dir        d2             d1    test_dir __file__
        test_dir = os.path.dirname(os.path.abspath(__file__))
        d1 = os.path.dirname(test_dir)
        d2 = os.path.dirname(d1)
        top_dir = os.path.dirname(d2)
        return (test_dir, top_dir)

    def register_opts(self, conf):
        return
