package controller

import (
	"opendev.org/x/networking-spp/operator/pkg/controller/spp"
)

func init() {
	// AddToManagerFuncs is a list of functions to create controllers and add them to a manager.
	AddToManagerFuncs = append(AddToManagerFuncs, spp.Add)
}
