// +build !ignore_autogenerated

// Code generated by operator-sdk. DO NOT EDIT.

package v1

import (
	runtime "k8s.io/apimachinery/pkg/runtime"
)

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *ClassifierTableOption) DeepCopyInto(out *ClassifierTableOption) {
	*out = *in
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new ClassifierTableOption.
func (in *ClassifierTableOption) DeepCopy() *ClassifierTableOption {
	if in == nil {
		return nil
	}
	out := new(ClassifierTableOption)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *ClassifierTableResponse) DeepCopyInto(out *ClassifierTableResponse) {
	*out = *in
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new ClassifierTableResponse.
func (in *ClassifierTableResponse) DeepCopy() *ClassifierTableResponse {
	if in == nil {
		return nil
	}
	out := new(ClassifierTableResponse)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *ComponentOption) DeepCopyInto(out *ComponentOption) {
	*out = *in
	if in.RxPort != nil {
		in, out := &in.RxPort, &out.RxPort
		*out = make([]PortOption, len(*in))
		copy(*out, *in)
	}
	if in.TxPort != nil {
		in, out := &in.TxPort, &out.TxPort
		*out = make([]PortOption, len(*in))
		copy(*out, *in)
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new ComponentOption.
func (in *ComponentOption) DeepCopy() *ComponentOption {
	if in == nil {
		return nil
	}
	out := new(ComponentOption)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *ComponentResponse) DeepCopyInto(out *ComponentResponse) {
	*out = *in
	if in.RxPort != nil {
		in, out := &in.RxPort, &out.RxPort
		*out = make([]PortResponse, len(*in))
		copy(*out, *in)
	}
	if in.TxPort != nil {
		in, out := &in.TxPort, &out.TxPort
		*out = make([]PortResponse, len(*in))
		copy(*out, *in)
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new ComponentResponse.
func (in *ComponentResponse) DeepCopy() *ComponentResponse {
	if in == nil {
		return nil
	}
	out := new(ComponentResponse)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *EalOption) DeepCopyInto(out *EalOption) {
	*out = *in
	if in.Vdevs != nil {
		in, out := &in.Vdevs, &out.Vdevs
		*out = make([]VdevOption, len(*in))
		for i := range *in {
			if (*in)[i] != nil {
				in, out := &(*in)[i], &(*out)[i]
				*out = make(VdevOption, len(*in))
				copy(*out, *in)
			}
		}
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new EalOption.
func (in *EalOption) DeepCopy() *EalOption {
	if in == nil {
		return nil
	}
	out := new(EalOption)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *PatchOption) DeepCopyInto(out *PatchOption) {
	*out = *in
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new PatchOption.
func (in *PatchOption) DeepCopy() *PatchOption {
	if in == nil {
		return nil
	}
	out := new(PatchOption)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *PortOption) DeepCopyInto(out *PortOption) {
	*out = *in
	out.Vlan = in.Vlan
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new PortOption.
func (in *PortOption) DeepCopy() *PortOption {
	if in == nil {
		return nil
	}
	out := new(PortOption)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *PortResponse) DeepCopyInto(out *PortResponse) {
	*out = *in
	out.Vlan = in.Vlan
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new PortResponse.
func (in *PortResponse) DeepCopy() *PortResponse {
	if in == nil {
		return nil
	}
	out := new(PortResponse)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in ResourceOption) DeepCopyInto(out *ResourceOption) {
	{
		in := &in
		*out = make(ResourceOption, len(*in))
		copy(*out, *in)
		return
	}
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new ResourceOption.
func (in ResourceOption) DeepCopy() ResourceOption {
	if in == nil {
		return nil
	}
	out := new(ResourceOption)
	in.DeepCopyInto(out)
	return *out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *Spp) DeepCopyInto(out *Spp) {
	*out = *in
	out.TypeMeta = in.TypeMeta
	in.ObjectMeta.DeepCopyInto(&out.ObjectMeta)
	in.Spec.DeepCopyInto(&out.Spec)
	in.Status.DeepCopyInto(&out.Status)
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new Spp.
func (in *Spp) DeepCopy() *Spp {
	if in == nil {
		return nil
	}
	out := new(Spp)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyObject is an autogenerated deepcopy function, copying the receiver, creating a new runtime.Object.
func (in *Spp) DeepCopyObject() runtime.Object {
	if c := in.DeepCopy(); c != nil {
		return c
	}
	return nil
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppApp) DeepCopyInto(out *SppApp) {
	*out = *in
	if in.Command != nil {
		in, out := &in.Command, &out.Command
		*out = make([]string, len(*in))
		copy(*out, *in)
	}
	if in.Args != nil {
		in, out := &in.Args, &out.Args
		*out = make([]string, len(*in))
		copy(*out, *in)
	}
	in.Eal.DeepCopyInto(&out.Eal)
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppApp.
func (in *SppApp) DeepCopy() *SppApp {
	if in == nil {
		return nil
	}
	out := new(SppApp)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppCtl) DeepCopyInto(out *SppCtl) {
	*out = *in
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppCtl.
func (in *SppCtl) DeepCopy() *SppCtl {
	if in == nil {
		return nil
	}
	out := new(SppCtl)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppList) DeepCopyInto(out *SppList) {
	*out = *in
	out.TypeMeta = in.TypeMeta
	in.ListMeta.DeepCopyInto(&out.ListMeta)
	if in.Items != nil {
		in, out := &in.Items, &out.Items
		*out = make([]Spp, len(*in))
		for i := range *in {
			(*in)[i].DeepCopyInto(&(*out)[i])
		}
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppList.
func (in *SppList) DeepCopy() *SppList {
	if in == nil {
		return nil
	}
	out := new(SppList)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyObject is an autogenerated deepcopy function, copying the receiver, creating a new runtime.Object.
func (in *SppList) DeepCopyObject() runtime.Object {
	if c := in.DeepCopy(); c != nil {
		return c
	}
	return nil
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppMirror) DeepCopyInto(out *SppMirror) {
	*out = *in
	in.Eal.DeepCopyInto(&out.Eal)
	if in.Components != nil {
		in, out := &in.Components, &out.Components
		*out = make([]ComponentOption, len(*in))
		for i := range *in {
			(*in)[i].DeepCopyInto(&(*out)[i])
		}
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppMirror.
func (in *SppMirror) DeepCopy() *SppMirror {
	if in == nil {
		return nil
	}
	out := new(SppMirror)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppNfv) DeepCopyInto(out *SppNfv) {
	*out = *in
	in.Eal.DeepCopyInto(&out.Eal)
	if in.Resources != nil {
		in, out := &in.Resources, &out.Resources
		*out = make(ResourceOption, len(*in))
		copy(*out, *in)
	}
	if in.Patches != nil {
		in, out := &in.Patches, &out.Patches
		*out = make([]PatchOption, len(*in))
		copy(*out, *in)
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppNfv.
func (in *SppNfv) DeepCopy() *SppNfv {
	if in == nil {
		return nil
	}
	out := new(SppNfv)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppNfvResponse) DeepCopyInto(out *SppNfvResponse) {
	*out = *in
	if in.Resources != nil {
		in, out := &in.Resources, &out.Resources
		*out = make([]string, len(*in))
		copy(*out, *in)
	}
	if in.Patches != nil {
		in, out := &in.Patches, &out.Patches
		*out = make([]map[string]string, len(*in))
		for i := range *in {
			if (*in)[i] != nil {
				in, out := &(*in)[i], &(*out)[i]
				*out = make(map[string]string, len(*in))
				for key, val := range *in {
					(*out)[key] = val
				}
			}
		}
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppNfvResponse.
func (in *SppNfvResponse) DeepCopy() *SppNfvResponse {
	if in == nil {
		return nil
	}
	out := new(SppNfvResponse)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppPcap) DeepCopyInto(out *SppPcap) {
	*out = *in
	in.Eal.DeepCopyInto(&out.Eal)
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppPcap.
func (in *SppPcap) DeepCopy() *SppPcap {
	if in == nil {
		return nil
	}
	out := new(SppPcap)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppPcapResponse) DeepCopyInto(out *SppPcapResponse) {
	*out = *in
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppPcapResponse.
func (in *SppPcapResponse) DeepCopy() *SppPcapResponse {
	if in == nil {
		return nil
	}
	out := new(SppPcapResponse)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppPrimary) DeepCopyInto(out *SppPrimary) {
	*out = *in
	in.Eal.DeepCopyInto(&out.Eal)
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppPrimary.
func (in *SppPrimary) DeepCopy() *SppPrimary {
	if in == nil {
		return nil
	}
	out := new(SppPrimary)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppProcessResponse) DeepCopyInto(out *SppProcessResponse) {
	*out = *in
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppProcessResponse.
func (in *SppProcessResponse) DeepCopy() *SppProcessResponse {
	if in == nil {
		return nil
	}
	out := new(SppProcessResponse)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppSpec) DeepCopyInto(out *SppSpec) {
	*out = *in
	out.Ctl = in.Ctl
	in.Primary.DeepCopyInto(&out.Primary)
	if in.Nfvs != nil {
		in, out := &in.Nfvs, &out.Nfvs
		*out = make([]SppNfv, len(*in))
		for i := range *in {
			(*in)[i].DeepCopyInto(&(*out)[i])
		}
	}
	if in.Vfs != nil {
		in, out := &in.Vfs, &out.Vfs
		*out = make([]SppVf, len(*in))
		for i := range *in {
			(*in)[i].DeepCopyInto(&(*out)[i])
		}
	}
	if in.Mirrors != nil {
		in, out := &in.Mirrors, &out.Mirrors
		*out = make([]SppMirror, len(*in))
		for i := range *in {
			(*in)[i].DeepCopyInto(&(*out)[i])
		}
	}
	if in.Pcaps != nil {
		in, out := &in.Pcaps, &out.Pcaps
		*out = make([]SppPcap, len(*in))
		for i := range *in {
			(*in)[i].DeepCopyInto(&(*out)[i])
		}
	}
	if in.Apps != nil {
		in, out := &in.Apps, &out.Apps
		*out = make([]SppApp, len(*in))
		for i := range *in {
			(*in)[i].DeepCopyInto(&(*out)[i])
		}
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppSpec.
func (in *SppSpec) DeepCopy() *SppSpec {
	if in == nil {
		return nil
	}
	out := new(SppSpec)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppStatus) DeepCopyInto(out *SppStatus) {
	*out = *in
	out.Ctl = in.Ctl
	out.Primary = in.Primary
	in.Nfvs.DeepCopyInto(&out.Nfvs)
	in.Vfs.DeepCopyInto(&out.Vfs)
	in.Mirrors.DeepCopyInto(&out.Mirrors)
	in.Pcaps.DeepCopyInto(&out.Pcaps)
	in.Apps.DeepCopyInto(&out.Apps)
	if in.Events != nil {
		in, out := &in.Events, &out.Events
		*out = make([]string, len(*in))
		copy(*out, *in)
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppStatus.
func (in *SppStatus) DeepCopy() *SppStatus {
	if in == nil {
		return nil
	}
	out := new(SppStatus)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppStatusApp) DeepCopyInto(out *SppStatusApp) {
	*out = *in
	if in.Pods != nil {
		in, out := &in.Pods, &out.Pods
		*out = make([]SppStatusPodBasic, len(*in))
		copy(*out, *in)
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppStatusApp.
func (in *SppStatusApp) DeepCopy() *SppStatusApp {
	if in == nil {
		return nil
	}
	out := new(SppStatusApp)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppStatusBasic) DeepCopyInto(out *SppStatusBasic) {
	*out = *in
	out.Pods = in.Pods
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppStatusBasic.
func (in *SppStatusBasic) DeepCopy() *SppStatusBasic {
	if in == nil {
		return nil
	}
	out := new(SppStatusBasic)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppStatusNfv) DeepCopyInto(out *SppStatusNfv) {
	*out = *in
	if in.Pods != nil {
		in, out := &in.Pods, &out.Pods
		*out = make(map[string]*SppStatusPodNfv, len(*in))
		for key, val := range *in {
			var outVal *SppStatusPodNfv
			if val == nil {
				(*out)[key] = nil
			} else {
				in, out := &val, &outVal
				*out = new(SppStatusPodNfv)
				(*in).DeepCopyInto(*out)
			}
			(*out)[key] = outVal
		}
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppStatusNfv.
func (in *SppStatusNfv) DeepCopy() *SppStatusNfv {
	if in == nil {
		return nil
	}
	out := new(SppStatusNfv)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppStatusPcap) DeepCopyInto(out *SppStatusPcap) {
	*out = *in
	if in.Pods != nil {
		in, out := &in.Pods, &out.Pods
		*out = make(map[string]*SppStatusPodPcap, len(*in))
		for key, val := range *in {
			var outVal *SppStatusPodPcap
			if val == nil {
				(*out)[key] = nil
			} else {
				in, out := &val, &outVal
				*out = new(SppStatusPodPcap)
				(*in).DeepCopyInto(*out)
			}
			(*out)[key] = outVal
		}
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppStatusPcap.
func (in *SppStatusPcap) DeepCopy() *SppStatusPcap {
	if in == nil {
		return nil
	}
	out := new(SppStatusPcap)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppStatusPodBasic) DeepCopyInto(out *SppStatusPodBasic) {
	*out = *in
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppStatusPodBasic.
func (in *SppStatusPodBasic) DeepCopy() *SppStatusPodBasic {
	if in == nil {
		return nil
	}
	out := new(SppStatusPodBasic)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppStatusPodNfv) DeepCopyInto(out *SppStatusPodNfv) {
	*out = *in
	if in.Response != nil {
		in, out := &in.Response, &out.Response
		*out = new(SppNfvResponse)
		(*in).DeepCopyInto(*out)
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppStatusPodNfv.
func (in *SppStatusPodNfv) DeepCopy() *SppStatusPodNfv {
	if in == nil {
		return nil
	}
	out := new(SppStatusPodNfv)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppStatusPodPcap) DeepCopyInto(out *SppStatusPodPcap) {
	*out = *in
	if in.Response != nil {
		in, out := &in.Response, &out.Response
		*out = new(SppPcapResponse)
		**out = **in
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppStatusPodPcap.
func (in *SppStatusPodPcap) DeepCopy() *SppStatusPodPcap {
	if in == nil {
		return nil
	}
	out := new(SppStatusPodPcap)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppStatusPodVf) DeepCopyInto(out *SppStatusPodVf) {
	*out = *in
	if in.Response != nil {
		in, out := &in.Response, &out.Response
		*out = new(SppVfResponse)
		(*in).DeepCopyInto(*out)
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppStatusPodVf.
func (in *SppStatusPodVf) DeepCopy() *SppStatusPodVf {
	if in == nil {
		return nil
	}
	out := new(SppStatusPodVf)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppStatusVf) DeepCopyInto(out *SppStatusVf) {
	*out = *in
	if in.Pods != nil {
		in, out := &in.Pods, &out.Pods
		*out = make(map[string]*SppStatusPodVf, len(*in))
		for key, val := range *in {
			var outVal *SppStatusPodVf
			if val == nil {
				(*out)[key] = nil
			} else {
				in, out := &val, &outVal
				*out = new(SppStatusPodVf)
				(*in).DeepCopyInto(*out)
			}
			(*out)[key] = outVal
		}
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppStatusVf.
func (in *SppStatusVf) DeepCopy() *SppStatusVf {
	if in == nil {
		return nil
	}
	out := new(SppStatusVf)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppVf) DeepCopyInto(out *SppVf) {
	*out = *in
	in.Eal.DeepCopyInto(&out.Eal)
	if in.Components != nil {
		in, out := &in.Components, &out.Components
		*out = make([]ComponentOption, len(*in))
		for i := range *in {
			(*in)[i].DeepCopyInto(&(*out)[i])
		}
	}
	if in.ClassifierTable != nil {
		in, out := &in.ClassifierTable, &out.ClassifierTable
		*out = make([]ClassifierTableOption, len(*in))
		copy(*out, *in)
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppVf.
func (in *SppVf) DeepCopy() *SppVf {
	if in == nil {
		return nil
	}
	out := new(SppVf)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *SppVfResponse) DeepCopyInto(out *SppVfResponse) {
	*out = *in
	if in.Ports != nil {
		in, out := &in.Ports, &out.Ports
		*out = make([]string, len(*in))
		copy(*out, *in)
	}
	if in.Components != nil {
		in, out := &in.Components, &out.Components
		*out = make([]ComponentResponse, len(*in))
		for i := range *in {
			(*in)[i].DeepCopyInto(&(*out)[i])
		}
	}
	if in.ClassifierTable != nil {
		in, out := &in.ClassifierTable, &out.ClassifierTable
		*out = make([]ClassifierTableResponse, len(*in))
		copy(*out, *in)
	}
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new SppVfResponse.
func (in *SppVfResponse) DeepCopy() *SppVfResponse {
	if in == nil {
		return nil
	}
	out := new(SppVfResponse)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in VdevOption) DeepCopyInto(out *VdevOption) {
	{
		in := &in
		*out = make(VdevOption, len(*in))
		copy(*out, *in)
		return
	}
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new VdevOption.
func (in VdevOption) DeepCopy() VdevOption {
	if in == nil {
		return nil
	}
	out := new(VdevOption)
	in.DeepCopyInto(out)
	return *out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *VlanOption) DeepCopyInto(out *VlanOption) {
	*out = *in
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new VlanOption.
func (in *VlanOption) DeepCopy() *VlanOption {
	if in == nil {
		return nil
	}
	out := new(VlanOption)
	in.DeepCopyInto(out)
	return out
}

// DeepCopyInto is an autogenerated deepcopy function, copying the receiver, writing into out. in must be non-nil.
func (in *VlanResponse) DeepCopyInto(out *VlanResponse) {
	*out = *in
	return
}

// DeepCopy is an autogenerated deepcopy function, copying the receiver, creating a new VlanResponse.
func (in *VlanResponse) DeepCopy() *VlanResponse {
	if in == nil {
		return nil
	}
	out := new(VlanResponse)
	in.DeepCopyInto(out)
	return out
}
